-- $Id$ 

-- mini test for the parser. will use that to test each
-- new feature

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


ENTITY conditions IS
END ENTITY;
--

-- test architecture declaration
ARCHITECTURE cond_imp of conditions IS
	variable x : integer;
	variable b : boolean;
BEGIN
	tc : process
	begin
		if 1 = 2 then
			null;
		else
			null;
		end if;

		if x = 2 then
		elsif x = 3 then
			null;
		elsif x = 4 then
			null;
		end if;


		if b then
			null;
		end if;
	end process tc;

end ARCHITECTURE cond_imp;
