/* $Id$ 
 *
 * Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#include "frontend/ast/Expression.hpp"
#include "frontend/ast/SubtypeIndication.hpp"

namespace ast {

Expression::~Expression()
{
	util::MiscUtil::terminate(this->type);
}

}; /* namespace ast */
