-- $Id$ 

-- check use-clauses 

-- Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


library mylib;
use mylib.desc;
library typedefs;
use typedefs.standard.integer;

Entity dummy IS
	signal dummy2 : integer;
end Entity dummy;

library typedefs;
use typedefs.standard;
architecture impl of dummy is
	signal somebool : standard.boolean;
begin
	p : process(dummy2) 
	begin
		somebool <= dummy.dummy2;
	end process;
end architecture impl;
