-- $Id: wait.vhdl 4327 2009-01-27 14:41:18Z potyra $ 

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

ENTITY test_bench IS
END ENTITY test_bench;

ARCHITECTURE tb of test_bench is
begin
	p : process
		constant a : string := """";
		constant b : string := "a""b";
		constant c : string := "a""""b"; 
	begin
		-- should be ""
		assert false report a severity note;
		assert a(0) = '"' report "double quotes 1" severity failure;
		-- should be a"b
		assert false report b severity note;
		--assert b(1) = '"' report "double quotes 2" severity failure;
		-- should be a""b
		--assert false report c severity note;
		--assert c(1) = '"' report "double quotes 3a" severity failure;
		--assert c(2) = '"' report "double quotes 3b" severity failure;

		assert false report "simulation finished" severity note;
		wait;
	end process p;

END ARCHITECTURE tb;
