-- $Id$ 

-- test a for loop 

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


entity test_bench is
end entity test_bench;

architecture test_bench_impl of test_bench is
begin
	bar : process
	variable a : integer;
	begin
		a := 1;
		for i in 1 to 2 loop
			a := a * 2;
		end loop;
		assert a = 4 report "wrong iteration count" severity failure;
		assert false report "simulation finished" severity note;
		wait;
	end process bar;
end architecture test_bench_impl;
