/* $Id$
 * vim:tabstop=8:shiftwidth=8:filetype=cpp:textwidth=72:
 *
 * Base class Visitor (intermediate code). Template implementations.
 *
 * Copyright (C) 2010 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

namespace intermediate {

template <typename T>
void
Visitor::listTraverse(T &l)
{
	for (typename T::const_iterator i = l.begin(); i != l.end(); i++) {
		(*i)->accept(*this);
	}
}

}; /* namespace intermediate */
