-- $Id$ 

-- small test case for attributes.

-- Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


library typedefs;
use typedefs.standard.all;

Entity desc IS
	signal foo : integer;
END Entity desc;

architecture implementation of desc is
	signal bar : integer;	
begin
	P : process
	begin
		wait until foo'event;
		bar <= foo'quiet(4);
	end process P;
end desc;
