-- $Id$ 

-- a process with a sensitivity list mustn't contain a wait statement, variant
-- via a procedure call.

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package procs is 
	procedure dostuff(signal i : in integer);
end package procs;

package body procs is
	procedure dostuff(signal i : in integer) is
	begin
		wait on i;
	end;
end procs;

use procs.dostuff;
entity something is
end entity something;

architecture implementation of something is
	signal s : integer;
begin
	x : process(s)
		variable a : integer;
	begin
		a := 1;
		dostuff(s); -- @ERROR@ Wait statement via procedure
	end process;
end implementation;
