-- $Id$ 

-- small test initializing an array of an array with an aggregate

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package t is 
	type byte is array(0 to 7) of bit;
	type word is array(0 to 3) of byte;
end package t;

entity something is
end entity something;

use t.word;
architecture implementation of something is
begin
	x : process
		variable w : word;
	begin
		w := (0 => "10011001", 1 | 3 => "00010001", 
			others => ( 4 to 7 => '0', others => '1'));
	end process;
end implementation;
