/* $Id$ 
 *
 * Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#ifndef __GARBAGE_COLLECT_HPP_INCLUDED
#define __GARBAGE_COLLECT_HPP_INCLUDED


#if HAVE_GC_GC_CPP_H
#include <gc/gc_cpp.h>
#endif /* HAVE_GC_GC_CPP_H */

namespace util {

#if HAVE_GC_GC_CPP_H
class GarbageCollect : public gc::gc {
public:
	//! initialize garbage collection.
	static void initialize(void) {
		GC_INIT();
	}

protected:
	/** virtual d'tor */
	virtual ~GarbageCollect() {}
};

#else /* ! HAVE_GC_GC_CPP_H */

class GarbageCollect {
public:
	//! dummy initialization, do nothing.
	static void initialize(void) {}

protected:
	//! virtual d'tor
	virtual ~GarbageCollect() {}
};

#endif /* HAVE_GC_GC_CPP_H */

}; /* namespace util */

#endif /* __GARBAGE_COLLECT_HPP_INCLUDED */
