-- $Id$ 

-- small test of the "foreign" attribute, with two type errors

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package types is
	type sig_boolean is (true, false);
	attribute foreign of sig_boolean : type is 1; -- @ERROR@ Type error
end package types;

use types.all;
entity fentity is
	PORT (p : in sig_boolean);
	attribute foreign of fentity : entity is false; -- @ERROR@ Type error
end entity fentity;
	
use types.all;
entity test_bench is
end entity test_bench;

architecture test_bench_impl of test_bench is
	signal fsig : sig_boolean;
begin
	f : fentity PORT MAP (p => fsig);

	x : process
	begin
		assert false report "simulation finished" severity note;
		wait;
	end process;
end test_bench_impl;
