-- $Id$ 

-- test a for loop

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package t is 
	type byte is array(1 to 8) of bit;
end package t;

entity foo is
end entity foo;

use t.byte;
architecture foo_impl of foo is
begin
	bar : process
	variable a : byte;
	begin
		for i in 1 to 8 loop
			a(i) := '1';
		end loop;
	end process bar;
end architecture foo_impl;
