/* $Id$ 
 * vim:tabstop=8:shiftwidth=8:filetype=cpp:textwidth=72:
 * Visitor: base class for all visitors.
 *
 * Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#include <iostream>
#include <sstream>
#include <cassert>

namespace ast {

template <typename T>
void
Visitor::listTraverse(const T &l)
{
	for (typename T::const_iterator i = l.begin(); i != l.end(); i++) {
		assert(*i);
		(*i)->accept(*this);
	}
}

template <typename T>
void
Visitor::listTraverse(T &l, bool &deleteFlag)
{
	typename T::iterator i=l.begin();
	
	while (i != l.end()) {
		assert(*i);
		(*i)->accept(*this);
		
		if (deleteFlag) {
			i = l.erase(i);
			deleteFlag = false;
			continue;
		}
		i++;
	}
}

template <typename T>
void
Visitor::listTraverse(T &l, bool &replaceFlag, T &replace)
{
	typename T::iterator i=l.begin();

	while (i != l.end()) {
		assert(*i);
		(*i)->accept(*this);

		if (replaceFlag) {
			i = l.erase(i);
			l.insert(i, replace.begin(), replace.end());

			replace.clear();
			replaceFlag = false;
			continue;
		}
		i++;
	}
}

template <typename T>
void
Visitor::listTraverse(T &l, T &insert)
{
	for (typename T::iterator i = l.begin(); i != l.end(); i++) {
		assert(*i);
		(*i)->accept(*this);

		if (! insert.empty()) {
			l.insert(i, insert.begin(), insert.end());
			insert.clear();
		}
	}
}

}; /* namespace ast */
