-- $Id$ 

-- Copyright (C) 2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

entity test_bench is
end entity test_bench;

architecture test_bench_impl of test_bench is
	signal s : boolean;
begin
	p : process
	begin
		-- FIXME needs proper attribute name handling.
		--if s = false and s'event then
		if s and s'event then
			assert false report "event" severity failure;
		end if;
		s <= true;
		wait on s until s'event;
		if not s'event then
			assert false report "event 2" severity failure;
		end if;

		assert false report "simulation finished" severity note;
		wait;
	end process;
end test_bench_impl;
