-- $Id$ 

-- define some basic types in packages std (to be used with initially empty
-- symbol table

-- Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package standard is 
	type BOOLEAN is (FALSE, TRUE);
	type BIT is ('0', '1');
	type INTEGER is range 0 to 2147483647;
	type NATURAL is range 1 to 10;
	type SEVERITY_LEVEL is (note, warning, error, failure);
end package standard;
