-- $Id$ 

-- small subscription test 1

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package t is 
	type byte is array(0 to 7) of bit;
end package t;

entity test_bench is
end entity test_bench;

use t.byte;
architecture test_bench_impl of test_bench is
begin
	x : process
		variable b : byte;
		variable a : byte;
	begin
		a := "00010001";
		b := "11101110";
		b := a;
		assert b(0)='0' report "array fail" severity failure;
		assert b(1)='0' report "array fail" severity failure;
		assert b(2)='0' report "array fail" severity failure;
		assert b(3)='1' report "array fail" severity failure;
		assert b(4)='0' report "array fail" severity failure;
		assert b(5)='0' report "array fail" severity failure;
		assert b(6)='0' report "array fail" severity failure;
		assert b(7)='1' report "array fail" severity failure;
		assert false report "simulation finished" severity note;
		wait;
	end process;
end test_bench_impl;
