-- $Id$
--
--  assemble an aggregate with a wrong number of indices.
--  should yield a type error.

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

package t is 
	type small_byte is array(0 to 6) of bit;
	type large_byte is array(0 to 8) of bit;
end package t;

entity something is
end entity something;

use t.small_byte;
use t.large_byte;
architecture implementation of something is
begin
	x : process
		variable a : small_byte;
		variable b : large_byte;
	begin
		a := "10011001"; -- @ERROR@ Index out of range
		b := "10011001"; -- @ERROR@ Indices not covered
	end process;
end implementation;
