/* $Id$
 * AttributeSpecification: contains the combination of AttributeSpecification 
 * to it's initializer Expression (AST node).
 *
 * Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#include "frontend/ast/AttributeSpecification.hpp"

namespace ast {

void
AttributeSpecification::put(std::ostream &stream) const
{
	stream << "ATTRIBUTE SPEC of " << *this->declaration
		<< " IS " << *this->init;
}

}; /* namespace ast */
