-- $Id$

-- test initializing of an unconstraint array works.

-- Copyright (C) 2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


ENTITY test_bench IS
END ENTITY test_bench;

ARCHITECTURE test_bench_impl of test_bench IS
	constant s : string := "simulation finished";
begin
	p : process
	begin
		wait for 10 ms;
		assert false report s severity note;
		wait;
	
	end process;

end test_bench_impl;
