-- $Id$ 

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

entity foo is
end entity foo;

architecture implementation of foo is
begin
	p : process
		type mystring is array(integer range <>) of character;
		-- FIXME doesn't work atm, see ResolveTypes (ValDeclaration).
		--variable x : mystring := "hallo welt";
		type byte_a is array(0 to 7) of bit;
		type word_a is array(0 to 1) of byte_a;
		variable mem_a : word_a;

		type word_b is array(0 to 1, 0 to 7) of bit;
		variable mem_b : word_b;
	begin
		mem_a := (0 => "10000001", 1 => "00011000");
		mem_a := (0 => (0 => '1', others => '0'), 
			  1 => (3 | 4 => '1', others => '0'));
		mem_a := (0 => (0 => '1', others => '0'), 
			  1 => (3 to 4 | 5 => '1', others => '0'));
		mem_b := (0 => (0 => '1', others => '0'), 
			  1 => (3 | 4 => '1', others => '0'));
		mem_b := (0 => (0 => '1', others => '0'), 
			  1 => (3 to 4 | 5 => '1', others => '0'));
	end process;
	
end implementation;
