-- $Id$ 

-- attribute names test 1

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package t is 
	type byte is array(0 to 7) of bit;
end package t;

entity test_bench is
end entity test_bench;

use t.all;
architecture test_bench_impl of test_bench is
	variable b : byte;
begin
	p : process
	begin
		b := "00000000";
		for i in b'range loop
			b(i) := '1';
		end loop;

		for j in 0 to 7 loop
			assert b(j) = '1' report "loop fail" severity failure;
		end loop;

		assert false report "simulation finished" severity note;
		wait;
	end process; 
end test_bench_impl;
