/* $Id$
 * Interpreter library: resolve labels/references.
 *
 * Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#ifndef __LOOKUP_SYMBOLS_H_INCLUDED
#define __LOOKUP_SYMBOLS_H_INCLUDED

#include "fauhdli_private.h"

extern void
fauhdli_resolve_symbols(struct fauhdli *instance);

#endif /* __LOOKUP_SYMBOLS_H_INCLUDED */
