-- $Id$ 

-- check if visibility of a port hides a type with the same name.

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


entity test_bench is
	PORT (boolean : in boolean;
		something : out boolean); -- @ERROR@ Parse error
end entity test_bench;
