-- $Id$ 

-- loop w.o. until clause

-- Copyright (C) 2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

entity test_bench is
end entity test_bench;

architecture test_bench_impl of test_bench is
begin
	p : process
	begin
		loop
			assert false report "looping" severity note;
			exit;
		end loop;
		assert false report "simulation finished" severity note;
		wait;
	end process;
end test_bench_impl;
