/* Compiler.hpp: Compatibility layer for compilers.
 *
 * Copyright (C) 2017 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */
#ifndef __COMPILER_HPP_INLCUDED
#define __COMPILER_HPP_INLCUDED

/* attribute fallthrough is supported from gcc-7 onwards */
#if __GNUC__ > 6
#define ATTRIBUTE_FALLTHROUGH __attribute__((__fallthrough__));
#else
#define ATTRIBUTE_FALLTHROUGH /* nothing */
#endif

#endif /* __COMPILER_HPP_INLCUDED */
