/* $Id$
 *
 * AtributeDeclaration: AST Class representing an attribute declaration.
 *
 * Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#include "frontend/ast/AttributeDeclaration.hpp"
#include "frontend/ast/TypeDeclaration.hpp"

namespace ast {

void 
AttributeDeclaration::put(std::ostream &stream) const
{
	assert(this->name != NULL);
	assert(this->type != NULL);
	stream << "ATTRIBUTE " << *this->name << ':' << *this->type;
}

}; /* namespace ast */
