-- $Id$ 

-- test a for while loop with a constant false condition.
-- this will "only" produce a warning, so it should be executed with -Werror

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


entity test_bench is
end entity test_bench;

architecture test_bench_impl of test_bench is
begin
	p : process
	begin
		while 41 - 5 * 8 > 100 / (5 + 5) loop -- @ERROR@ While-loop will
			assert false report "cannot reach loop." severity
			failure;
		end loop;

		assert false report "simulation finished" severity note;
		wait;
	end process p;
end architecture test_bench_impl;
