-- $Id$

-- downto test.

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

entity test_bench is
end entity test_bench;

architecture test_bench_impl of test_bench is
	variable a : string(17 downto 10);
begin
	x : process
	begin
		a := "76543210";
		assert a(10) = '0' report "error index 10" severity failure;
		assert a(11) = '1' report "error index 11" severity failure;
		assert a(12) = '2' report "error index 12" severity failure;
		assert a(13) = '3' report "error index 13" severity failure;
		assert a(14) = '4' report "error index 14" severity failure;
		assert a(15) = '5' report "error index 15" severity failure;
		assert a(16) = '6' report "error index 16" severity failure;
		assert a(17) = '7' report "error index 17" severity failure;
		assert false report "simulation finished" severity note;
		wait;
	end process;
end test_bench_impl;
