/* $Id$ 
 * Driver: one driver associated to a signal name.
 *
 * Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#include "frontend/misc/Driver.hpp"
#include "frontend/ast/SimpleName.hpp"

namespace ast {

Driver::Driver(
	const SignalDeclaration &d,
	unsigned int counter,
	SimpleName *reference
	) : 	signal(d),
		n(reference),
		cnt(counter) 
{}


Driver::~Driver()
{}

};
