#!/bin/sh
# $Id$
#
# convenience script to setup autotools.
# 
# Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
# This program is free software. You can redistribute it and/or modify it
# under the terms of the GNU General Public License, either version 2 of
# the License, or (at your option) any later version. See COPYING.

set -e

# create scripts directory in case it doesn't exist yet
if [ ! -d scripts ]; then
	mkdir -p scripts
fi

autoreconf -i
echo
echo "Now run ./configure to proceed in building FAUhdlc."
