-- $Id$ 

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

ENTITY unitA IS
	PORT (
		p1 : inout integer;
		p2 : in integer;
		p3 : out bit
	);
END ENTITY unitA;

ARCHITECTURE impl of unitA is
	attribute foreign of impl : architecture is "foobar";
begin
end;


ENTITY unitB IS 
END ENTITY unitB;

ARCHITECTURE implementation of unitB is
	SIGNAL s1 : integer;
	SIGNAL s2 : integer;
	SIGNAL s3 : bit;
BEGIN
	instUnit : unitA 
		PORT MAP (
			p2 => s2,
			p1 => s1,
			p3 => s3
		);
END implementation;
