-- $Id$ 

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

entity e is
end entity e;

architecture a of e is
	signal i : integer;
begin
	-- named process
	named_process : process
		variable x : integer := 0;
	begin
		i <= x;
		wait on i;
		x := i;
	end process named_process;
end a;
