-- $Id$ 

-- Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

Entity desc IS
	signal foo : integer;
END Entity desc;

architecture implementation of desc is
	signal bar : integer;
	
	function myFunc(x : integer) return integer is 
		variable bar : integer; -- homograph to imp.bar
	begin
		foo <= bar;
		foo <= x + bar;
		return 0;
	end;
begin
end desc;
