-- $Id$

-- assign a variable of a an array to a variable of an array with a different
-- direction

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

package t is
	type b_arr is array(integer range <>) of bit;
end package t;

entity test_bench is
end entity test_bench;

use t.b_arr;
architecture test_bench_impl of test_bench is
	variable a : b_arr(1 to 2);
	variable b : b_arr(2 downto 1);
begin
	x : process
	begin
		a := "10";
		b := a;
		assert b(2) = '1' report "error" severity failure;
		assert b(1) = '0' report "error" severity failure;
		assert false report "simulation finished" severity note;
		wait;
	end process;
end test_bench_impl;
