/* $Id$ 
 *
 * Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#ifndef __CONCURRENT_STAT_HPP_INCLUDED
#define __CONCURRENT_STAT_HPP_INCLUDED

#include "frontend/ast/SymbolDeclaration.hpp"

namespace ast {

/** abstract base class for all concurrent statements
 */
class ConcurrentStat : public SymbolDeclaration {
public:

	//! c'tor
	/** @param loc location of the concurrent statement
         */
	ConcurrentStat(
		Location loc
		) : 	SymbolDeclaration(NULL, loc) {}
};

};

#endif /* __CONCURRENT_STAT_HPP_INCLUDED */
