-- $Id$ 

-- small function call test.

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package funcs is
	function foo(a, b : integer) return integer;
end package funcs;

package body funcs is
	function foo(a, b : integer) return integer is
	begin
		return a + b;
	end;
end;

	
use funcs.foo;
entity test_bench is
end entity test_bench;

architecture test_bench_impl of test_bench is
begin
	x : process
		variable a, b, x : integer;
	begin
		a := 1;
		b := 2;
		x := foo(a, b);
		assert(x = 3) report "x != 3" severity failure;
		assert false report "simulation finished" severity note;
		wait;
	end process;
end test_bench_impl;
