-- $Id$ 

-- test a while loop.

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


entity foo is
end entity foo;

architecture foo_impl of foo is
begin
	bar : process
	variable i : integer;
	begin
		i := 1;
		while i < 500 loop
			i := i * 2 + 3;
		end loop;
	end process bar;
end architecture foo_impl;
