-- $Id$
--
--  subscribe to a multimensional array with only one index and vice versa.
--
-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package t is 
	type rom128 is array(0 to 128, 0 to 7) of bit;
	type byte is array(0 to 7) of bit;
end package t;

entity something is
end entity something;

use t.rom128;
use t.byte;
architecture implementation of something is
begin
	x : process
		variable a : byte;
		variable b : rom128;
	begin
		b(8) := '1';	-- @ERROR@ too few indices
		a(8, 2) := '0'; -- @ERROR@ too many indices
	end process;
end implementation;
