-- $Id$ 

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.

package t is 
	procedure foo(constant a : in string);
end package t;

package body t is
	procedure foo(constant a : in string) is
	begin
		assert false report a severity note;
	end;
end t;

entity test_bench is
end entity test_bench;

use t.all;
architecture test_bench_impl of test_bench is
begin
	p : process
	begin
		foo("");
		foo("simulation finished");
		wait;
	end process;
end test_bench_impl;
