-- $Id$ 

-- small test for record types

-- Copyright (C) 2008-2009 FAUmachine Team <info@faumachine.org>.
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.


package t is 
	type byte is array(0 to 7) of bit;
	type rec is record 
		elem1 : integer;
		elem2 : byte;
	end record;

end package t;

entity something is
end entity something;

use t.rec;
use t.byte;
architecture implementation of something is
begin
	x : process
		variable r : rec;
		variable b : byte;
	begin
		r.elem2 := b;
		r.elem1 := 1;
	end process;
end implementation;
